@@ -1,6 +1,6 @@
 ;;; lang/c99/mach.scm - C parser grammer
 
-;; Copyright (C) 2015-2020 Matthew R. Wette
+;; Copyright (C) 2015-2021 Matthew R. Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -37,8 +37,7 @@
   #:use-module (nyacc lalr)
   #:use-module (nyacc parse)
   #:use-module (nyacc lex)
-  #:use-module (nyacc util)
-  #:use-module ((srfi srfi-43) #:select (vector-map)))
+  #:use-module (nyacc util))
 
 ;; @deffn {Variable} c99-spec
 ;; This variable is the specification a-list for the hacked ISO C99 language.
@@ -110,7 +109,10 @@
      ("--" unary-expression ($$ `(pre-dec ,$2)))
      (unary-operator cast-expression ($$ (list $1 $2)))
      ("sizeof" unary-expression ($$ `(sizeof-expr ,$2)))
-     ("sizeof" "(" type-name ")" ($$ `(sizeof-type ,$3))))
+     ("sizeof" "(" type-name ")" ($$ `(sizeof-type ,$3)))
+     ("_Alignof" "(" type-name ")" ($$ `(alignof-type ,$3)))
+     ("__builtin_offsetof" "(" type-name "," constant-expression ")"
+      ($$ `(offsetof-type ,$3 ,$5))))
     
     (unary-operator ("&" ($$ 'ref-to)) ("*" ($$ 'de-ref))
 		    ("+" ($$ 'pos)) ("-" ($$ 'neg))
@@ -299,7 +301,8 @@
     (float-type-specifier
      ("float" ($prec 'imp) ($$ '(float-type "float")))
      ("double" ($prec 'imp) ($$ '(float-type "double")))
-     ("long" "double" ($$ '(float-type "long double"))))
+     ("long" "double" ($$ '(float-type "long double")))
+     ("_Float128" ($$ '(float-type "_Float128"))))
 
     (complex-type-specifier
      ("_Complex" ($$ '(complex-type "_Complex")))
